package com.sobot.fastjson;

import android.util.Log;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

//json 工具类
public class SobotJsonUtils {

    /**
     * 返回Json字符串里面包含的一个对象
     *
     * @param jsonStr  :{"city":"china","map1":[{"age":"28","name":"yys"}]}
     * @param list_str :map1
     * @param clazz    :Map.class,或者其他对象
     * @param <T>      :Map
     * @return :List<Map>
     */
    public static final <T> List<T> jsonToChildList(String jsonStr, String list_str,
                                                    Class<T> clazz) {
        JSONObject jsonobj = JSON.parseObject(jsonStr);
        if (jsonobj == null) {
            return null;
        }
        Object obj = jsonobj.get(list_str);
        if (obj == null) {
            return null;
        }
        if (obj instanceof JSONArray) {
            JSONArray jsonarr = (JSONArray) obj;
            List<T> list = new ArrayList<T>();
            for (int i = 0; i < jsonarr.size(); i++) {
                list.add(jsonarr.getObject(i, clazz));
            }
            return list;
        }
        return null;
    }

    /**
     * 返回Json字符串里面包含的一个对象
     *
     * @param jsonStr :{"department":{"id":"1","name":"生产部"},"password":"admin","username":"admin"}
     * @param obj_str :department
     * @param clazz   :Map.class,或者其他对象
     * @param <T>     :Map
     * @return
     */
    public static final <T> T jsonToChildObj(String jsonStr, String obj_str,
                                             Class<T> clazz) {
        JSONObject jsonobj = JSON.parseObject(jsonStr);
        if (jsonobj == null) {
            return null;
        }

        Object obj = jsonobj.get(obj_str);
        if (obj == null) {
            return null;
        }

        if (obj instanceof JSONObject) {
            return jsonobj.getObject(obj_str, clazz);
        } else {
            Log.i("", obj.getClass() + "");
        }

        return null;
    }

    /**
     * json 转换成对象
     *
     * @param jsonStr
     * @param clazz
     * @param <T>
     * @return
     */
    public static final <T> T jsonToObj(String jsonStr, Class<T> clazz) {
        T t = null;
        try {
            t = JSON.parseObject(jsonStr, clazz);
        } catch (Exception e) {
            Log.e("json字符串转换失败！" + jsonStr, e.getMessage());
        }
        return t;
    }

    /**
     * 对象转换成json字符串(带有格式化输出)
     *
     * @param object       要转换的对象
     * @param prettyFormat 是否格式化json字符串,输出带有换行和缩进的字符串
     * @return 返回一个json 字符串数组
     */
    public static final String objToJson(Object object, boolean prettyFormat) {
        return JSON.toJSONString(object, prettyFormat);
    }


    /**
     * 对象转换成json字符串
     *
     * @param object 要转换的对象
     * @return 返回一个json 字符串数组
     */
    public static final String objToJson(Object object) {
        return JSON.toJSONString(object);
    }

    /**
     * json 字符串转换成原始的Object对象
     *
     * @param jsonStr
     * @return
     */
    public static final Object jsonToObj(String jsonStr) {
        return JSON.parse(jsonStr);
    }

    /**
     * json字符串转换成list
     *
     * @param jsonStr    Json字符串
     * @param clazz      要转换的class
     * @param <T>返回的泛型类型
     * @return 返回的<T>泛型类型
     */
    public static <T> List<T> jsonToList(String jsonStr, Class<T> clazz) {
        List<T> list = new ArrayList<T>();
        try {
            list = JSON.parseArray(jsonStr, clazz);
        } catch (Exception e) {
            Log.e("json字符串转List失败！" + jsonStr, e.toString());
        }
        return list;
    }

    /**
     * json字符串转换成Map
     *
     * @param jsonStr Json字符串
     * @return Map
     */
    public static Map<String, Object> jsonToMap(String jsonStr) {
        try {
            return JSON.parseObject(jsonStr, Map.class);
        } catch (Exception e) {
            Log.e("json字符串转换失败！" + jsonStr, e.getMessage());
        }
        return null;
    }

    /**
     * json 转换成list<map>
     *
     * @param jsonString
     * @return
     */
    public static List<Map<String, Object>> jsonToListkeyMap(String jsonString) {
        List<Map<String, Object>> list = new ArrayList<Map<String, Object>>();
        try {
            list = JSON.parseObject(jsonString,
                    new TypeReference<List<Map<String, Object>>>() {
                    });
        } catch (Exception e) {
            Log.e("json字符串转map失败", e.getMessage());
        }
        return list;
    }
}
